import { DataObject, Filter, ListModel, PageData, RESTClient } from 'mobx-restful';
import { Constructor } from 'web-utility';
import { UserIdType } from '../../type';
import { BITable, TableCellLink, TableCellRelation, TableCellText, TableRecord, TableRecordFields, TableView } from './type';
export * from './type';
export type FilterOperator = '<' | '<=' | '=' | '!=' | '=>' | '>' | 'contains';
/**
 * @see https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/bitable-v1/filter
 */
export declare function makeSimpleFilter(data: DataObject, operator?: FilterOperator, relation?: 'AND' | 'OR'): string;
export declare const normalizeText: (value: TableCellText | TableCellLink | TableCellRelation) => string;
export type BiBaseData = Omit<TableRecord<{}>, 'record_id' | 'fields'>;
export interface BiDataQueryOptions {
    text_field_as_array?: boolean;
    automatic_fields?: boolean;
    display_formula_ref?: boolean;
    with_shared_url?: boolean;
    user_id_type?: UserIdType;
}
/**
 * @see {@link https://open.feishu.cn/document/ukTMukTMukTM/uUDN04SN0QjL1QDN/bitable-overview}
 */
export declare function BiDataTable<T extends DataObject, F extends Filter<T> = Filter<T>>(Base?: typeof ListModel): abstract new (appId: string, tableId: string) => {
    requiredKeys: readonly (keyof T)[];
    sort: Partial<Record<keyof T, "ASC" | "DESC">>;
    queryOptions: BiDataQueryOptions;
    currentViewId?: string;
    normalize({ fields, ...meta }: TableRecord<T>): T;
    /**
     * @see {@link https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/bitable-v1/app-table-record/get}
     */
    getOne(id: string): Promise<T>;
    /**
     * @see {@link https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/bitable-v1/app-table-record/create}
     * @see {@link https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/bitable-v1/app-table-record/update}
     */
    updateOne(fields: F, id?: string): Promise<T>;
    makeFilter(filter: F): string;
    /**
     * @see {@link https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/bitable-v1/app-table-record/list}
     */
    openStream(filter: F): AsyncGenerator<T, void, unknown>;
    getViewList(viewId: string, pageIndex?: number, pageSize?: number): Promise<T[]>;
    getViewAll(viewId: string, pageSize?: number): Promise<T[]>;
    stream?: AsyncGenerator<T, void, any> | undefined;
    clearList: (() => void) & (() => void);
    clear: (() => void) & (() => void);
    restoreList: (({ filter, pageIndex, pageSize, allItems, totalCount }?: Partial<Pick<ListModel<T, Partial<import("mobx-restful").NewData<T>>>, "filter" | "pageIndex" | "pageSize" | "allItems" | "totalCount">> | undefined) => Promise<void>) & (({ pageIndex, pageSize, allItems, totalCount }?: Partial<Pick<ListModel<T, Partial<import("mobx-restful").NewData<T>>>, "pageIndex" | "pageSize" | "allItems" | "totalCount">> | undefined) => void);
    loadStream(filter: F, newCount: number): Promise<T[]>;
    loadPage: ((pageIndex: number, pageSize: number, filter: F) => Promise<{
        pageData: T[];
        totalCount: number;
    }>) & ((pageIndex: number, pageSize: number, filter: F) => Promise<PageData<T>>);
    pageIndex: number;
    pageSize: number;
    filter: F;
    totalCount: number | undefined;
    pageList: T[][];
    readonly currentPage: T[];
    readonly pageCount: number;
    readonly allItems: T[];
    readonly noMore: boolean;
    statistic: Partial<Record<import("web-utility").TypeKeys<T, import("web-utility").IndexKey>, Record<string, number>>>;
    turnTo: ((pageIndex: number, pageSize?: number) => ListModel<T, F>) & ((pageIndex: number, pageSize?: number) => ListModel<T, F>);
    loadNewPage: ((pageIndex: number, pageSize: number, filter: F) => Promise<{
        pageData: T[];
        totalCount: number;
    }>) & ((pageIndex: number, pageSize: number, filter: F) => Promise<{
        pageData: T[];
        totalCount: number;
    }>);
    getList: ((filter?: F | undefined, pageIndex?: number, pageSize?: number) => Promise<T[]>) & ((filter?: F | undefined, pageIndex?: number, pageSize?: number) => Promise<T[]>);
    refreshList: (() => Promise<T[]>) & (() => Promise<T[]>);
    getAll: ((filter?: F | undefined, pageSize?: number) => Promise<T[]>) & ((filter?: F | undefined, pageSize?: number) => Promise<T[]>);
    countAll: ((keys: import("web-utility").TypeKeys<T, import("web-utility").IndexKey>[], filter?: F | undefined, pageSize?: number) => Promise<Partial<Record<import("web-utility").TypeKeys<T, import("web-utility").IndexKey>, Record<string, number>>>>) & ((keys: import("web-utility").TypeKeys<T, import("web-utility").IndexKey>[], filter?: F | undefined, pageSize?: number) => Promise<Partial<Record<import("web-utility").TypeKeys<T, import("web-utility").IndexKey>, Record<string, number>>>>);
    indexOf: ((id: import("mobx-restful").IDType) => number) & ((id: import("mobx-restful").IDType) => number);
    changeOne: ((data: Partial<T>, id: import("mobx-restful").IDType, patch?: boolean) => void) & ((data: Partial<T>, id: import("mobx-restful").IDType, patch?: boolean) => void);
    removeOne: ((id: import("mobx-restful").IDType) => Promise<void>) & ((id: import("mobx-restful").IDType) => Promise<void>);
    deleteOne: ((id: import("mobx-restful").IDType) => Promise<void>) & ((id: import("mobx-restful").IDType) => Promise<void>);
    client: import("mobx-restful/dist/utility").RESTClient;
    baseURI: string;
    indexKey: keyof T;
    currentOne: T;
    clearCurrent: (() => void) & (() => void);
    downloading: number;
    uploading: number;
};
export declare function BiSearch<D extends DataObject, F extends Filter<D> = Filter<D>>(Model: Constructor<ListModel<D, F>>): abstract new (...args: any[]) => {
    baseURI: string;
    client: RESTClient;
    loadPage: (pageIndex: number, pageSize: number, filter: F) => Promise<PageData<D>>;
    searchKeys: readonly (keyof TableRecordFields)[];
    makeFilter(filter: F): string;
    getSearchList(keywords: string, pageIndex?: number, pageSize?: number): Promise<D[]>;
    pageIndex: number;
    pageSize: number;
    filter: F;
    totalCount: number | undefined;
    pageList: D[][];
    readonly currentPage: D[];
    readonly pageCount: number;
    readonly allItems: D[];
    readonly noMore: boolean;
    statistic: Partial<Record<import("web-utility").TypeKeys<D, import("web-utility").IndexKey>, Record<string, number>>>;
    clearList(): void;
    clear(): void;
    restoreList({ pageIndex, pageSize, allItems, totalCount }?: Partial<Pick<ListModel<D, Partial<import("mobx-restful").NewData<D>>>, "pageIndex" | "pageSize" | "allItems" | "totalCount">> | undefined): void;
    turnTo(pageIndex: number, pageSize?: number): ListModel<D, F>;
    loadNewPage(pageIndex: number, pageSize: number, filter: F): Promise<{
        pageData: D[];
        totalCount: number;
    }>;
    getList(filter?: F | undefined, pageIndex?: number, pageSize?: number): Promise<D[]>;
    refreshList(): Promise<D[]>;
    getAll(filter?: F | undefined, pageSize?: number): Promise<D[]>;
    countAll(keys: import("web-utility").TypeKeys<D, import("web-utility").IndexKey>[], filter?: F | undefined, pageSize?: number): Promise<Partial<Record<import("web-utility").TypeKeys<D, import("web-utility").IndexKey>, Record<string, number>>>>;
    indexOf(id: import("mobx-restful").IDType): number;
    changeOne(data: Partial<D>, id: import("mobx-restful").IDType, patch?: boolean): void;
    updateOne(data: Partial<import("mobx-restful").NewData<D>>, id?: import("mobx-restful").IDType): Promise<D>;
    removeOne(id: import("mobx-restful").IDType): Promise<void>;
    deleteOne(id: import("mobx-restful").IDType): Promise<void>;
    indexKey: keyof D;
    currentOne: D;
    clearCurrent(): void;
    getOne(id: import("mobx-restful").IDType): Promise<D>;
    downloading: number;
    uploading: number;
};
interface BiSearchModel extends InstanceType<ReturnType<typeof BiSearch<TableRecordFields, any>>> {
}
export type BiSearchModelClass = Constructor<BiSearchModel>;
export declare function BiTableView(): abstract new (appId: string, tableId: string) => {
    /**
     * @see {@link https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/bitable-v1/app-table-view/list}
     */
    openStream(): AsyncGenerator<TableView, void, unknown>;
    stream?: AsyncGenerator<TableView, void, any> | undefined;
    clearList: (() => void) & (() => void);
    clear: (() => void) & (() => void);
    restoreList: (({ filter, pageIndex, pageSize, allItems, totalCount }?: Partial<Pick<ListModel<TableView, Partial<import("mobx-restful").NewData<TableView>>>, "filter" | "pageIndex" | "pageSize" | "allItems" | "totalCount">> | undefined) => Promise<void>) & (({ pageIndex, pageSize, allItems, totalCount }?: Partial<Pick<ListModel<TableView, Partial<import("mobx-restful").NewData<TableView>>>, "pageIndex" | "pageSize" | "allItems" | "totalCount">> | undefined) => void);
    loadStream(filter: Partial<import("mobx-restful").NewData<TableView>>, newCount: number): Promise<TableView[]>;
    loadPage: ((pageIndex: number, pageSize: number, filter: Partial<import("mobx-restful").NewData<TableView>>) => Promise<{
        pageData: TableView[];
        totalCount: number;
    }>) & ((pageIndex: number, pageSize: number, filter: Partial<import("mobx-restful").NewData<TableView>>) => Promise<PageData<TableView>>);
    pageIndex: number;
    pageSize: number;
    filter: Partial<import("mobx-restful").NewData<TableView>>;
    totalCount: number | undefined;
    pageList: TableView[][];
    readonly currentPage: TableView[];
    readonly pageCount: number;
    readonly allItems: TableView[];
    readonly noMore: boolean;
    statistic: Partial<Record<import("web-utility").TypeKeys<TableView, import("web-utility").IndexKey>, Record<string, number>>>;
    turnTo: ((pageIndex: number, pageSize?: number) => ListModel<TableView, Partial<import("mobx-restful").NewData<TableView>>>) & ((pageIndex: number, pageSize?: number) => ListModel<TableView, Partial<import("mobx-restful").NewData<TableView>>>);
    loadNewPage: ((pageIndex: number, pageSize: number, filter: Partial<import("mobx-restful").NewData<TableView>>) => Promise<{
        pageData: TableView[];
        totalCount: number;
    }>) & ((pageIndex: number, pageSize: number, filter: Partial<import("mobx-restful").NewData<TableView>>) => Promise<{
        pageData: TableView[];
        totalCount: number;
    }>);
    getList: ((filter?: Partial<import("mobx-restful").NewData<TableView>> | undefined, pageIndex?: number, pageSize?: number) => Promise<TableView[]>) & ((filter?: Partial<import("mobx-restful").NewData<TableView>> | undefined, pageIndex?: number, pageSize?: number) => Promise<TableView[]>);
    refreshList: (() => Promise<TableView[]>) & (() => Promise<TableView[]>);
    getAll: ((filter?: Partial<import("mobx-restful").NewData<TableView>> | undefined, pageSize?: number) => Promise<TableView[]>) & ((filter?: Partial<import("mobx-restful").NewData<TableView>> | undefined, pageSize?: number) => Promise<TableView[]>);
    countAll: ((keys: import("web-utility").TypeKeys<TableView, import("web-utility").IndexKey>[], filter?: Partial<import("mobx-restful").NewData<TableView>> | undefined, pageSize?: number) => Promise<Partial<Record<import("web-utility").TypeKeys<TableView, import("web-utility").IndexKey>, Record<string, number>>>>) & ((keys: import("web-utility").TypeKeys<TableView, import("web-utility").IndexKey>[], filter?: Partial<import("mobx-restful").NewData<TableView>> | undefined, pageSize?: number) => Promise<Partial<Record<import("web-utility").TypeKeys<TableView, import("web-utility").IndexKey>, Record<string, number>>>>);
    indexOf: ((id: import("mobx-restful").IDType) => number) & ((id: import("mobx-restful").IDType) => number);
    changeOne: ((data: Partial<TableView>, id: import("mobx-restful").IDType, patch?: boolean) => void) & ((data: Partial<TableView>, id: import("mobx-restful").IDType, patch?: boolean) => void);
    updateOne: ((data: Partial<import("mobx-restful").NewData<TableView>>, id?: import("mobx-restful").IDType) => Promise<TableView>) & ((data: Partial<import("mobx-restful").NewData<TableView>>, id?: import("mobx-restful").IDType) => Promise<TableView>);
    removeOne: ((id: import("mobx-restful").IDType) => Promise<void>) & ((id: import("mobx-restful").IDType) => Promise<void>);
    deleteOne: ((id: import("mobx-restful").IDType) => Promise<void>) & ((id: import("mobx-restful").IDType) => Promise<void>);
    client: import("mobx-restful/dist/utility").RESTClient;
    baseURI: string;
    indexKey: keyof TableView;
    currentOne: TableView;
    clearCurrent: (() => void) & (() => void);
    getOne: ((id: import("mobx-restful").IDType) => Promise<TableView>) & ((id: import("mobx-restful").IDType) => Promise<TableView>);
    downloading: number;
    uploading: number;
};
export type BiDataTableClass<T extends DataObject, F extends Filter<T> = Filter<T>> = ReturnType<typeof BiDataTable<T, F>>;
export declare function BiTable(): abstract new (id: string) => {
    id: string;
    currentDataTable?: ListModel<any>;
    getOne<T extends DataObject, F extends Partial<import("mobx-restful").NewData<T>>>(tableName: string, DataTableClass?: (abstract new (appId: string, tableId: string) => {
        requiredKeys: readonly (keyof T)[];
        sort: Partial<Record<keyof T, "ASC" | "DESC">>;
        queryOptions: BiDataQueryOptions;
        currentViewId?: string;
        normalize({ fields, ...meta }: TableRecord<T>): T;
        /**
         * @see {@link https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/bitable-v1/app-table-record/get}
         */
        getOne(id: string): Promise<T>;
        /**
         * @see {@link https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/bitable-v1/app-table-record/create}
         * @see {@link https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/bitable-v1/app-table-record/update}
         */
        updateOne(fields: F, id?: string): Promise<T>;
        makeFilter(filter: F): string;
        /**
         * @see {@link https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/bitable-v1/app-table-record/list}
         */
        openStream(filter: F): AsyncGenerator<T, void, unknown>;
        getViewList(viewId: string, pageIndex?: number, pageSize?: number): Promise<T[]>;
        getViewAll(viewId: string, pageSize?: number): Promise<T[]>;
        stream?: AsyncGenerator<T, void, any> | undefined;
        clearList: (() => void) & (() => void);
        clear: (() => void) & (() => void);
        restoreList: (({ filter, pageIndex, pageSize, allItems, totalCount }?: Partial<Pick<ListModel<T, Partial<import("mobx-restful").NewData<T>>>, "filter" | "pageIndex" | "pageSize" | "allItems" | "totalCount">> | undefined) => Promise<void>) & (({ pageIndex, pageSize, allItems, totalCount }?: Partial<Pick<ListModel<T, Partial<import("mobx-restful").NewData<T>>>, "pageIndex" | "pageSize" | "allItems" | "totalCount">> | undefined) => void);
        loadStream(filter: F, newCount: number): Promise<T[]>;
        loadPage: ((pageIndex: number, pageSize: number, filter: F) => Promise<{
            pageData: T[];
            totalCount: number;
        }>) & ((pageIndex: number, pageSize: number, filter: F) => Promise<PageData<T>>);
        pageIndex: number;
        pageSize: number;
        filter: F;
        totalCount: number | undefined;
        pageList: T[][];
        readonly currentPage: T[];
        readonly pageCount: number;
        readonly allItems: T[];
        readonly noMore: boolean;
        statistic: Partial<Record<import("web-utility").TypeKeys<T, import("web-utility").IndexKey>, Record<string, number>>>;
        turnTo: ((pageIndex: number, pageSize?: number) => ListModel<T, F>) & ((pageIndex: number, pageSize?: number) => ListModel<T, F>);
        loadNewPage: ((pageIndex: number, pageSize: number, filter: F) => Promise<{
            pageData: T[];
            totalCount: number;
        }>) & ((pageIndex: number, pageSize: number, filter: F) => Promise<{
            pageData: T[];
            totalCount: number;
        }>);
        getList: ((filter?: F | undefined, pageIndex?: number, pageSize?: number) => Promise<T[]>) & ((filter?: F | undefined, pageIndex?: number, pageSize?: number) => Promise<T[]>);
        refreshList: (() => Promise<T[]>) & (() => Promise<T[]>);
        getAll: ((filter?: F | undefined, pageSize?: number) => Promise<T[]>) & ((filter?: F | undefined, pageSize?: number) => Promise<T[]>);
        countAll: ((keys: import("web-utility").TypeKeys<T, import("web-utility").IndexKey>[], filter?: F | undefined, pageSize?: number) => Promise<Partial<Record<import("web-utility").TypeKeys<T, import("web-utility").IndexKey>, Record<string, number>>>>) & ((keys: import("web-utility").TypeKeys<T, import("web-utility").IndexKey>[], filter?: F | undefined, pageSize?: number) => Promise<Partial<Record<import("web-utility").TypeKeys<T, import("web-utility").IndexKey>, Record<string, number>>>>);
        indexOf: ((id: import("mobx-restful").IDType) => number) & ((id: import("mobx-restful").IDType) => number);
        changeOne: ((data: Partial<T>, id: import("mobx-restful").IDType, patch?: boolean) => void) & ((data: Partial<T>, id: import("mobx-restful").IDType, patch?: boolean) => void);
        removeOne: ((id: import("mobx-restful").IDType) => Promise<void>) & ((id: import("mobx-restful").IDType) => Promise<void>);
        deleteOne: ((id: import("mobx-restful").IDType) => Promise<void>) & ((id: import("mobx-restful").IDType) => Promise<void>);
        client: import("mobx-restful/dist/utility").RESTClient;
        baseURI: string;
        indexKey: keyof T;
        currentOne: T;
        clearCurrent: (() => void) & (() => void);
        downloading: number;
        uploading: number;
    }) | undefined): Promise<BITable>;
    /**
     * @see {@link https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/bitable-v1/app-table/list}
     */
    openStream(): AsyncGenerator<BITable, void, unknown>;
    stream?: AsyncGenerator<BITable, void, any> | undefined;
    clearList: (() => void) & (() => void);
    clear: (() => void) & (() => void);
    restoreList: (({ filter, pageIndex, pageSize, allItems, totalCount }?: Partial<Pick<ListModel<BITable, Partial<import("mobx-restful").NewData<BITable>>>, "filter" | "pageIndex" | "pageSize" | "allItems" | "totalCount">> | undefined) => Promise<void>) & (({ pageIndex, pageSize, allItems, totalCount }?: Partial<Pick<ListModel<BITable, Partial<import("mobx-restful").NewData<BITable>>>, "pageIndex" | "pageSize" | "allItems" | "totalCount">> | undefined) => void);
    loadStream(filter: Partial<import("mobx-restful").NewData<BITable>>, newCount: number): Promise<BITable[]>;
    loadPage: ((pageIndex: number, pageSize: number, filter: Partial<import("mobx-restful").NewData<BITable>>) => Promise<{
        pageData: BITable[];
        totalCount: number;
    }>) & ((pageIndex: number, pageSize: number, filter: Partial<import("mobx-restful").NewData<BITable>>) => Promise<PageData<BITable>>);
    pageIndex: number;
    pageSize: number;
    filter: Partial<import("mobx-restful").NewData<BITable>>;
    totalCount: number | undefined;
    pageList: BITable[][];
    readonly currentPage: BITable[];
    readonly pageCount: number;
    readonly allItems: BITable[];
    readonly noMore: boolean;
    statistic: Partial<Record<import("web-utility").TypeKeys<BITable, import("web-utility").IndexKey>, Record<string, number>>>;
    turnTo: ((pageIndex: number, pageSize?: number) => ListModel<BITable, Partial<import("mobx-restful").NewData<BITable>>>) & ((pageIndex: number, pageSize?: number) => ListModel<BITable, Partial<import("mobx-restful").NewData<BITable>>>);
    loadNewPage: ((pageIndex: number, pageSize: number, filter: Partial<import("mobx-restful").NewData<BITable>>) => Promise<{
        pageData: BITable[];
        totalCount: number;
    }>) & ((pageIndex: number, pageSize: number, filter: Partial<import("mobx-restful").NewData<BITable>>) => Promise<{
        pageData: BITable[];
        totalCount: number;
    }>);
    getList: ((filter?: Partial<import("mobx-restful").NewData<BITable>> | undefined, pageIndex?: number, pageSize?: number) => Promise<BITable[]>) & ((filter?: Partial<import("mobx-restful").NewData<BITable>> | undefined, pageIndex?: number, pageSize?: number) => Promise<BITable[]>);
    refreshList: (() => Promise<BITable[]>) & (() => Promise<BITable[]>);
    getAll: ((filter?: Partial<import("mobx-restful").NewData<BITable>> | undefined, pageSize?: number) => Promise<BITable[]>) & ((filter?: Partial<import("mobx-restful").NewData<BITable>> | undefined, pageSize?: number) => Promise<BITable[]>);
    countAll: ((keys: import("web-utility").TypeKeys<BITable, import("web-utility").IndexKey>[], filter?: Partial<import("mobx-restful").NewData<BITable>> | undefined, pageSize?: number) => Promise<Partial<Record<import("web-utility").TypeKeys<BITable, import("web-utility").IndexKey>, Record<string, number>>>>) & ((keys: import("web-utility").TypeKeys<BITable, import("web-utility").IndexKey>[], filter?: Partial<import("mobx-restful").NewData<BITable>> | undefined, pageSize?: number) => Promise<Partial<Record<import("web-utility").TypeKeys<BITable, import("web-utility").IndexKey>, Record<string, number>>>>);
    indexOf: ((id: import("mobx-restful").IDType) => number) & ((id: import("mobx-restful").IDType) => number);
    changeOne: ((data: Partial<BITable>, id: import("mobx-restful").IDType, patch?: boolean) => void) & ((data: Partial<BITable>, id: import("mobx-restful").IDType, patch?: boolean) => void);
    updateOne: ((data: Partial<import("mobx-restful").NewData<BITable>>, id?: import("mobx-restful").IDType) => Promise<BITable>) & ((data: Partial<import("mobx-restful").NewData<BITable>>, id?: import("mobx-restful").IDType) => Promise<BITable>);
    removeOne: ((id: import("mobx-restful").IDType) => Promise<void>) & ((id: import("mobx-restful").IDType) => Promise<void>);
    deleteOne: ((id: import("mobx-restful").IDType) => Promise<void>) & ((id: import("mobx-restful").IDType) => Promise<void>);
    client: import("mobx-restful/dist/utility").RESTClient;
    baseURI: string;
    indexKey: keyof BITable;
    currentOne: BITable;
    clearCurrent: (() => void) & (() => void);
    downloading: number;
    uploading: number;
};
